@@ -136,6 +136,50 @@ finish_cb (GObject      *object,
     g_task_return_boolean (user_data, status);
 }
 
+static void
+settings_apply_style (GtkWidget  *widget,
+                      const char *style)
+{
+  GtkStyleContext *context;
+
+  g_assert (style && *style);
+
+  context = gtk_widget_get_style_context (widget);
+  gtk_style_context_add_class (context, style);
+}
+
+static void
+settings_remove_style (GtkWidget  *widget,
+                       const char *style)
+{
+  GtkStyleContext *context;
+
+  g_assert (style && *style);
+
+  context = gtk_widget_get_style_context (widget);
+  gtk_style_context_remove_class (context, style);
+}
+
+static void
+settings_check_librem_one (ChattySettingsDialog *self)
+{
+  GtkEntry *entry;
+  const char *server, *user_id;
+
+  entry = GTK_ENTRY (self->matrix_homeserver_entry);
+  server = gtk_entry_get_text (entry);
+  user_id = gtk_entry_get_text (GTK_ENTRY (self->new_account_id_entry));
+
+  if (!server || !*server) {
+    if (g_str_has_suffix (user_id, ":librem.one") ||
+        g_str_has_suffix (user_id, "@librem.one"))
+      gtk_entry_set_text (entry, "https://chat.librem.one");
+    else if (g_str_has_suffix (user_id, "talk.puri.sm") ||
+             g_str_has_suffix (user_id, "@puri.sm"))
+      gtk_entry_set_text (entry, "https://talk.puri.sm");
+  }
+}
+
 static void
 settings_dialog_set_save_state (ChattySettingsDialog *self,
                                 gboolean              in_progress)
@@ -312,22 +356,8 @@ chatty_settings_add_clicked_cb (ChattySettingsDialog *self)
   is_matrix = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->matrix_radio_button));
   is_telegram = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->telegram_radio_button));
 
-  if (is_matrix) {
-    GtkEntry *entry;
-    const char *server;
-
-    entry = GTK_ENTRY (self->matrix_homeserver_entry);
-    server = gtk_entry_get_text (entry);
-
-    if (!server || !*server) {
-      if (g_str_has_suffix (user_id, ":librem.one") ||
-          g_str_has_suffix (user_id, "@librem.one"))
-        gtk_entry_set_text (entry, "https://chat.librem.one");
-      else if (g_str_has_suffix (user_id, "talk.puri.sm") ||
-               g_str_has_suffix (user_id, "@puri.sm"))
-        gtk_entry_set_text (entry, "https://talk.puri.sm");
-    }
-  }
+  if (is_matrix)
+    settings_check_librem_one (self);
 
   if (is_matrix && chatty_settings_get_experimental_features (chatty_settings_get_default ())) {
     chatty_settings_save_matrix (self, user_id, password);
@@ -597,6 +627,11 @@ settings_homeserver_entry_changed (ChattySettingsDialog *self,
     valid = valid && *uri->host && g_str_equal (soup_uri_get_path (uri), "/");
   }
 
+  if (valid)
+    settings_remove_style (GTK_WIDGET (entry), "error");
+  else
+    settings_apply_style (GTK_WIDGET (entry), "error");
+
   gtk_widget_set_sensitive (self->add_button, valid);
 }
 
@@ -814,7 +849,7 @@ static void
 settings_new_detail_changed_cb (ChattySettingsDialog *self)
 {
   const gchar *id, *password;
-  ChattyProtocol protocol;
+  ChattyProtocol protocol, valid_protocol;
   gboolean valid = TRUE;
 
   g_assert (CHATTY_IS_SETTINGS_DIALOG (self));
@@ -822,6 +857,17 @@ settings_new_detail_changed_cb (ChattySettingsDialog *self)
   id = gtk_entry_get_text (GTK_ENTRY (self->new_account_id_entry));
   password = gtk_entry_get_text (GTK_ENTRY (self->new_password_entry));
 
+  if (password && *password)
+    settings_remove_style (GTK_WIDGET (self->new_password_entry), "error");
+  else
+    settings_apply_style (GTK_WIDGET (self->new_password_entry), "error");
+
+  if (!id || !*id) {
+    gtk_entry_set_text (GTK_ENTRY (self->matrix_homeserver_entry), "");
+    gtk_widget_show (self->matrix_homeserver_entry);
+    gtk_widget_hide (self->matrix_homeserver_entry);
+  }
+
   if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->matrix_radio_button)))
     protocol = CHATTY_PROTOCOL_MATRIX;
   else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->telegram_radio_button)))
@@ -833,12 +879,26 @@ settings_new_detail_changed_cb (ChattySettingsDialog *self)
       gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->matrix_radio_button)))
     protocol = CHATTY_PROTOCOL_MATRIX | CHATTY_PROTOCOL_EMAIL;
 
+  valid_protocol = chatty_utils_username_is_valid (id, protocol);
+  valid = valid && valid_protocol;
+
+  if (valid_protocol)
+    settings_remove_style (GTK_WIDGET (self->new_account_id_entry), "error");
+  else
+    settings_apply_style (GTK_WIDGET (self->new_account_id_entry), "error");
+
+  /* If user tried to login to matrix via email, ask for homeserver details */
+  if (protocol & CHATTY_PROTOCOL_MATRIX &&
+      valid_protocol & CHATTY_PROTOCOL_EMAIL) {
+    gtk_widget_show (self->matrix_homeserver_entry);
+    settings_check_librem_one (self);
+    settings_homeserver_entry_changed (self, GTK_ENTRY (self->matrix_homeserver_entry));
+  }
+
   /* Allow empty passwords for telegram accounts */
   if (protocol != CHATTY_PROTOCOL_TELEGRAM)
     valid = valid && password && *password;
 
-  valid = valid && chatty_utils_username_is_valid (id, protocol);
-
   /* Don’t allow adding if an account with same id exists */
   if (valid &&
       chatty_manager_find_account_with_name (chatty_manager_get_default (), protocol, id))
